@@ -15,42 +15,42 @@
 #include <config.h>
 #endif
 
-#include <gdk/gdkkeysyms.h>
 #include <assert.h>
 
-#include "fcintl.h"
+#include <gdk/gdkkeysyms.h>
+
 #include "events.h"
+#include "fcintl.h"
 #include "game.h"
 #include "mem.h"
 #include "support.h"
 
 #include "chatline_g.h"
 #include "citydlg_g.h"
+#include "civclient.h"
+#include "cma_fec.h"
+#include "messagewin_g.h"
+
 #include "cityrep.h"
 #include "dialogs.h"
-#include "inputdlg.h"
 #include "gui_stuff.h"
-#include "messagewin_g.h"
-#include "cma_fec.h"
+#include "helpdlg.h"
+#include "inputdlg.h"
 
 #include "cma_fe.h"
 
 #define BUFFER_SIZE             64
 
-#define SPECLIST_TAG cma_dialog
+#define SPECLIST_TAG dialog
 #define SPECLIST_TYPE struct cma_dialog
 #include "speclist.h"
 
-#define SPECLIST_TAG cma_dialog
-#define SPECLIST_TYPE struct cma_dialog
-#include "speclist_c.h"
+#define dialog_list_iterate(dialoglist, pdialog) \
+    TYPED_LIST_ITERATE(struct cma_dialog, dialoglist, pdialog)
+#define dialog_list_iterate_end  LIST_ITERATE_END
 
-#define cma_dialog_list_iterate(presetlist, ppreset) \
-    TYPED_LIST_ITERATE(struct cma_dialog, presetlist, ppreset)
-#define cma_dialog_list_iterate_end  LIST_ITERATE_END
-
-static struct cma_dialog_list cma_dialog_list;
-static int cma_dialog_list_has_been_initialised;
+static struct dialog_list dialog_list;
+static bool dialog_list_has_been_initialised = FALSE;
 
 static int allow_refreshes = 1;
 
@@ -68,27 +68,24 @@ static void cma_preset_remove_response(GtkWidget *w, gint response,
 static void cma_add_preset_callback(GtkWidget *w, gpointer data);
 static void cma_preset_add_callback_yes(GtkWidget *w, gpointer data);
 static void cma_preset_add_callback_no(GtkWidget *w, gpointer data);
-static int cma_preset_add_callback_delete(GtkWidget *w, GdkEvent *ev,
-					  gpointer data);
+static void cma_preset_add_callback_destroy(GtkWidget *w, gpointer data);
 
-static void cma_change_to_callback(GtkWidget *w, gpointer data);
-static void cma_change_permanent_to_callback(GtkWidget *w, gpointer data);
-static void cma_release_callback(GtkWidget *w, gpointer data);
-static void cma_select_preset_callback(GtkWidget *w, gint row, gint col,
-				       GdkEvent *ev, gpointer data);
+static void cma_active_callback(GtkWidget *w, gpointer data);
+static void cma_activate_preset_callback(GtkTreeView *view, GtkTreePath *path,
+				         GtkTreeViewColumn *col, gpointer data);
 
 static void hscale_changed(GtkAdjustment *get, gpointer data);
-static void set_hscales(const struct cma_parameter *const parameter,
+static void set_hscales(const struct cm_parameter *const parameter,
 			struct cma_dialog *pdialog);
 
 /**************************************************************************
 ...
 **************************************************************************/
-static void ensure_initialised_cma_dialog_list(void)
+static void ensure_initialised_dialog_list(void)
 {
-  if (!cma_dialog_list_has_been_initialised) {
-    cma_dialog_list_init(&cma_dialog_list);
-    cma_dialog_list_has_been_initialised = 1;
+  if (!dialog_list_has_been_initialised) {
+    dialog_list_init(&dialog_list);
+    dialog_list_has_been_initialised = TRUE;
   }
 }
 
@@ -99,9 +96,19 @@ void close_cma_dialog(struct city *pcity)
 {
   struct cma_dialog *pdialog = get_cma_dialog(pcity);
 
-  cma_dialog_list_unlink(&cma_dialog_list, pdialog);
-
   gtk_widget_destroy(pdialog->shell);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void cma_dialog_destroy_callback(GtkWidget *w, gpointer data)
+{
+  struct cma_dialog *pdialog = (struct cma_dialog *) data;
+
+  g_object_unref(pdialog->tips);
+
+  dialog_list_unlink(&dialog_list, pdialog);
   free(pdialog);
 }
 
@@ -110,81 +117,175 @@ void close_cma_dialog(struct city *pcity)
 *****************************************************************/
 struct cma_dialog *get_cma_dialog(struct city *pcity)
 {
-  ensure_initialised_cma_dialog_list();
+  ensure_initialised_dialog_list();
 
-  cma_dialog_list_iterate(cma_dialog_list, pdialog) {
+  dialog_list_iterate(dialog_list, pdialog) {
     if (pdialog->pcity == pcity) {
       return pdialog;
     }
-  } cma_dialog_list_iterate_end;
+  } dialog_list_iterate_end;
 
   return NULL;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev,
+				      gpointer data)
+{
+  GtkTreePath *path;
+  GtkTreeViewColumn *column;
+
+  if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(view),
+	ev->x, ev->y, &path, &column, NULL, NULL)) {
+    if (ev->type == GDK_BUTTON_PRESS) {
+      cma_activate_preset_callback(view, path, column, data);
+    } else if (ev->type == GDK_2BUTTON_PRESS) {
+      struct cma_dialog *pdialog = (struct cma_dialog *) data;
+      struct cm_parameter param;
+
+      cmafec_get_fe_parameter(pdialog->pcity, &param);
+      cma_put_city_under_agent(pdialog->pcity, &param);
+      refresh_city_dialog(pdialog->pcity);
+    }
+  }
+  gtk_tree_path_free(path);
+
+  return FALSE;
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void help_callback(GtkWidget *w, gpointer data)
+{
+  popup_help_dialog_string(HELP_CMA_ITEM);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *cell,
+			   GtkTreeModel *model, GtkTreeIter *it, gpointer data)
+{
+  struct cma_dialog *pdialog = (struct cma_dialog *) data;
+  const char *s1, *s2;
+  int i1, i2;
+  struct cm_parameter param;
+  GtkTreePath *path;
+
+  gtk_tree_model_get(model, it, 0, &s1, -1); 
+  path = gtk_tree_model_get_path(model, it);
+  i1 = gtk_tree_path_get_indices(path) [0];
+  gtk_tree_path_free(path);
+
+  cmafec_get_fe_parameter(pdialog->pcity, &param);
+  s2 = cmafec_get_short_descr(&param);
+  i2 = cmafec_preset_get_index_of_parameter(&param);
+
+  if (!strcmp(s1, s2) && i1 == i2) {
+    g_object_set(G_OBJECT(cell), "style", PANGO_STYLE_ITALIC,
+		 "weight", PANGO_WEIGHT_BOLD, NULL);
+  } else {
+    g_object_set(G_OBJECT(cell), "style", PANGO_STYLE_NORMAL,
+		 "weight", PANGO_WEIGHT_NORMAL, NULL);
+  }
+}
+
 /**************************************************************************
  instantiates a new struct for each city_dialog window that is open.
 **************************************************************************/
 struct cma_dialog *create_cma_dialog(struct city *pcity)
 {
   struct cma_dialog *pdialog;
-  struct cma_parameter param;
+  struct cm_parameter param;
   GtkWidget *frame, *page, *hbox, *label, *table;
-  GtkWidget *vbox, *scrolledwindow, *hscale;
-  static char *preset_title[] = { N_("Presets") };
-  static bool preset_title_done;
+  GtkWidget *vbox, *sw, *hscale, *button, *align;
   int i;
+  GtkListStore *store;
+  GtkCellRenderer *rend;
+  GtkWidget *view;
+  GtkTreeViewColumn *column;
 
   cmafec_get_fe_parameter(pcity, &param);
   pdialog = fc_malloc(sizeof(struct cma_dialog));
   pdialog->pcity = pcity;
-  pdialog->shell = gtk_vbox_new(FALSE, 0);
+  pdialog->shell = gtk_vbox_new(FALSE, 8);
+  gtk_container_set_border_width(GTK_CONTAINER(pdialog->shell), 8);
+  g_signal_connect(pdialog->shell, "destroy",
+		   G_CALLBACK(cma_dialog_destroy_callback), pdialog);
 
-  intl_slist(ARRAY_SIZE(preset_title), preset_title, &preset_title_done);
+  pdialog->tips = gtk_tooltips_new();
+  g_object_ref(pdialog->tips);
+  gtk_object_sink(GTK_OBJECT(pdialog->tips));
 
-  frame = gtk_frame_new(_("Citizen Management Agent"));
-  gtk_box_pack_start(GTK_BOX(pdialog->shell), frame, TRUE, TRUE, 0);
+  page = gtk_hbox_new(FALSE, 12);
+  gtk_box_pack_start(GTK_BOX(pdialog->shell), page, TRUE, TRUE, 0);
 
-  page = gtk_hbox_new(FALSE, 0);
-  gtk_container_add(GTK_CONTAINER(frame), page);
-
-  vbox = gtk_vbox_new(FALSE, 0);
+  vbox = gtk_vbox_new(FALSE, 2);
   gtk_box_pack_start(GTK_BOX(page), vbox, TRUE, TRUE, 0);
 
-  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
-  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
+  sw = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
+				      GTK_SHADOW_ETCHED_IN);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
-  pdialog->preset_list = gtk_clist_new_with_titles(1, preset_title);
-  gtk_container_add(GTK_CONTAINER(scrolledwindow), pdialog->preset_list);
-  gtk_clist_set_column_auto_resize(GTK_CLIST(pdialog->preset_list), 0, TRUE);
-  gtk_clist_column_titles_show(GTK_CLIST(pdialog->preset_list));
-  gtk_clist_column_titles_passive(GTK_CLIST(pdialog->preset_list));
+  store = gtk_list_store_new(1, G_TYPE_STRING);
+  pdialog->store = store;
 
-  gtk_signal_connect(GTK_OBJECT(pdialog->preset_list), "select_row",
-		     GTK_SIGNAL_FUNC(cma_select_preset_callback), pdialog);
+  view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+  g_object_unref(store);
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
+  pdialog->preset_list = view;
+  pdialog->selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
 
-  gtk_signal_connect(GTK_OBJECT(pdialog->preset_list), "key-press-event",
-		     GTK_SIGNAL_FUNC(cma_preset_key_pressed_callback),
-		     pdialog);
+  g_signal_connect(pdialog->preset_list, "button_press_event",
+      		   G_CALLBACK(button_press_callback), pdialog);
 
+  gtk_tooltips_set_tip(pdialog->tips, view,
+		       _("For information on:\n"
+		         "CMA and presets\n"
+			 "including sample presets,\n"
+		         "see README.cma."),
+		       "");
 
-  hbox = gtk_hbox_new(FALSE, 0);
+  rend = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(NULL, rend,
+      "text", 0, NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
+  gtk_tree_view_column_set_cell_data_func(column, rend, cell_data_func,
+      pdialog, NULL);
+
+  label = g_object_new(GTK_TYPE_LABEL,
+                       "use-underline", TRUE,
+                       "mnemonic-widget", view,
+                       "label", _("_Presets:"),
+                       "xalign", 0.0, "yalign", 0.5, NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+
+  gtk_container_add(GTK_CONTAINER(sw), view);
+  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
+
+  g_signal_connect(view, "row_activated",
+		   G_CALLBACK(cma_activate_preset_callback), pdialog);
+  g_signal_connect(view, "key-press-event",
+		   G_CALLBACK(cma_preset_key_pressed_callback), pdialog);
+
+  hbox = gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_EDGE);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  pdialog->add_preset_command = gtk_button_new_with_label(_("Add preset"));
-  gtk_box_pack_start(GTK_BOX(hbox), pdialog->add_preset_command,
-		     TRUE, TRUE, 0);
-  GTK_WIDGET_SET_FLAGS(pdialog->add_preset_command, GTK_CAN_DEFAULT);
-  gtk_signal_connect(GTK_OBJECT(pdialog->add_preset_command), "clicked",
-		     GTK_SIGNAL_FUNC(cma_add_preset_callback), pdialog);
+  pdialog->add_preset_command = gtk_button_new_from_stock(GTK_STOCK_NEW);
+  gtk_container_add(GTK_CONTAINER(hbox), pdialog->add_preset_command);
+  g_signal_connect(pdialog->add_preset_command, "clicked",
+		   G_CALLBACK(cma_add_preset_callback), pdialog);
 
-  pdialog->del_preset_command = gtk_button_new_with_label(_("Delete preset"));
-  gtk_box_pack_start(GTK_BOX(hbox), pdialog->del_preset_command,
-		     TRUE, TRUE, 0);
-  GTK_WIDGET_SET_FLAGS(pdialog->del_preset_command, GTK_CAN_DEFAULT);
-  gtk_signal_connect(GTK_OBJECT(pdialog->del_preset_command), "clicked",
-		     GTK_SIGNAL_FUNC(cma_del_preset_callback), pdialog);
+  pdialog->del_preset_command = gtk_button_new_from_stock(GTK_STOCK_DELETE);
+  gtk_container_add(GTK_CONTAINER(hbox), pdialog->del_preset_command);
+  g_signal_connect(pdialog->del_preset_command, "clicked",
+		   G_CALLBACK(cma_del_preset_callback), pdialog);
 
   /* the right-hand side */
 
@@ -204,7 +305,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
 
   /* Minimal Surplus and Factor */
 
-  table = gtk_table_new(NUM_STATS + 2, 3, FALSE);
+  table = gtk_table_new(NUM_STATS + 2, 3, TRUE);
   gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 2);
 
   label = gtk_label_new(_("Minimal Surplus"));
@@ -215,7 +316,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 0, 1);
 
   for (i = 0; i < NUM_STATS; i++) {
-    label = gtk_label_new(cma_get_stat_name(i));
+    label = gtk_label_new(cm_get_stat_name(i));
     gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i + 1, i + 2);
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
@@ -227,20 +328,20 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
     gtk_scale_set_digits(GTK_SCALE(hscale), 0);
     gtk_scale_set_value_pos(GTK_SCALE(hscale), GTK_POS_LEFT);
 
-    gtk_signal_connect(GTK_OBJECT(pdialog->minimal_surplus[i]),
-		       "value_changed", GTK_SIGNAL_FUNC(hscale_changed),
-		       pdialog);
+    g_signal_connect(pdialog->minimal_surplus[i],
+		     "value_changed",
+		     G_CALLBACK(hscale_changed), pdialog);
 
     pdialog->factor[i] =
-	GTK_ADJUSTMENT(gtk_adjustment_new(1, 1, 25, 1, 1, 0));
+	GTK_ADJUSTMENT(gtk_adjustment_new(1, 0, 25, 1, 1, 0));
 
     hscale = gtk_hscale_new(GTK_ADJUSTMENT(pdialog->factor[i]));
     gtk_table_attach_defaults(GTK_TABLE(table), hscale, 2, 3, i + 1, i + 2);
     gtk_scale_set_digits(GTK_SCALE(hscale), 0);
     gtk_scale_set_value_pos(GTK_SCALE(hscale), GTK_POS_LEFT);
 
-    gtk_signal_connect(GTK_OBJECT(pdialog->factor[i]), "value_changed",
-		       GTK_SIGNAL_FUNC(hscale_changed), pdialog);
+    g_signal_connect(pdialog->factor[i], "value_changed",
+		     G_CALLBACK(hscale_changed), pdialog);
   }
 
   /* Happy Surplus and Factor */
@@ -260,11 +361,11 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdialog->happy_button),
 			       FALSE);
 
-  gtk_signal_connect(GTK_OBJECT(pdialog->happy_button), "toggled",
-		     GTK_SIGNAL_FUNC(hscale_changed), pdialog);
+  g_signal_connect(pdialog->happy_button, "toggled",
+		   G_CALLBACK(hscale_changed), pdialog);
 
   pdialog->factor[NUM_STATS] =
-      GTK_ADJUSTMENT(gtk_adjustment_new(1, 1, 50, 1, 0, 0));
+      GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 50, 1, 0, 0));
 
   hscale = gtk_hscale_new(GTK_ADJUSTMENT(pdialog->factor[NUM_STATS]));
   gtk_table_attach_defaults(GTK_TABLE(table), hscale, 2, 3,
@@ -272,42 +373,47 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   gtk_scale_set_digits(GTK_SCALE(hscale), 0);
   gtk_scale_set_value_pos(GTK_SCALE(hscale), GTK_POS_LEFT);
 
-  gtk_signal_connect(GTK_OBJECT(pdialog->factor[NUM_STATS]),
-		     "value_changed", GTK_SIGNAL_FUNC(hscale_changed),
-		     pdialog);
+  g_signal_connect(pdialog->factor[NUM_STATS],
+		   "value_changed",
+		   G_CALLBACK(hscale_changed), pdialog);
 
   /* buttons */
 
-  hbox = gtk_hbox_new(FALSE, 0);
+  hbox = gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_EDGE);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  pdialog->change_command = gtk_button_new_with_mnemonic(_("Apply onc_e"));
-  gtk_box_pack_start(GTK_BOX(hbox), pdialog->change_command, TRUE, FALSE, 0);
-  GTK_WIDGET_SET_FLAGS(pdialog->change_command, GTK_CAN_DEFAULT);
-  gtk_signal_connect(GTK_OBJECT(pdialog->change_command), "clicked",
-		     GTK_SIGNAL_FUNC(cma_change_to_callback), pdialog);
+  button = gtk_button_new_from_stock(GTK_STOCK_HELP);
+  g_signal_connect(button, "clicked",
+		   G_CALLBACK(help_callback), NULL);
+  gtk_container_add(GTK_CONTAINER(hbox), button);
 
-  pdialog->perm_command = gtk_button_new_with_mnemonic(_("Control c_ity"));
-  gtk_box_pack_start(GTK_BOX(hbox), pdialog->perm_command, TRUE, FALSE, 0);
-  GTK_WIDGET_SET_FLAGS(pdialog->perm_command, GTK_CAN_DEFAULT);
-  gtk_signal_connect(GTK_OBJECT(pdialog->perm_command), "clicked",
-		     GTK_SIGNAL_FUNC(cma_change_permanent_to_callback),
-		     pdialog);
+  pdialog->active_command = gtk_toggle_button_new();
+  gtk_container_add(GTK_CONTAINER(hbox), pdialog->active_command);
 
-  pdialog->release_command = gtk_button_new_with_mnemonic(_("_Release city"));
-  gtk_box_pack_start(GTK_BOX(hbox), pdialog->release_command, TRUE, FALSE, 0);
-  GTK_WIDGET_SET_FLAGS(pdialog->release_command, GTK_CAN_DEFAULT);
-  gtk_signal_connect(GTK_OBJECT(pdialog->release_command), "clicked",
-		     GTK_SIGNAL_FUNC(cma_release_callback), pdialog);
+  align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
+  gtk_container_add(GTK_CONTAINER(pdialog->active_command), align);
+
+  vbox = gtk_vbox_new(FALSE, 2);
+  gtk_container_add(GTK_CONTAINER(align), vbox);
+
+  pdialog->active_image = gtk_image_new();
+  gtk_box_pack_start(GTK_BOX(vbox), pdialog->active_image, FALSE, FALSE, 0);
+
+  pdialog->active_label = gtk_label_new(NULL);
+  gtk_widget_set_name(pdialog->active_label, "comment label");
+  gtk_box_pack_end(GTK_BOX(vbox), pdialog->active_label, FALSE, FALSE, 0);
 
   gtk_widget_show_all(pdialog->shell);
 
-  ensure_initialised_cma_dialog_list();
+  ensure_initialised_dialog_list();
 
-  cma_dialog_list_insert(&cma_dialog_list, pdialog);
+  dialog_list_insert(&dialog_list, pdialog);
 
   update_cma_preset_list(pdialog);
 
+  gtk_tree_view_focus(GTK_TREE_VIEW(view));
+
   /* refresh is done in refresh_city_dialog */
 
   return pdialog;
@@ -318,42 +424,47 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
 **************************************************************************/
 void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
 {
-  struct cma_result result;
-  struct cma_parameter param;
+  struct cm_result result;
+  struct cm_parameter param;
   struct cma_dialog *pdialog = get_cma_dialog(pcity);
   int controlled = cma_is_city_under_agent(pcity, NULL);
-  int preset_index;
 
   cmafec_get_fe_parameter(pcity, &param);
 
   /* fill in result label */
-  cma_query_result(pcity, &param, &result);
-  gtk_set_label(pdialog->result_label,
-		(char *) cmafec_get_result_descr(pcity, &result, &param));
+  cm_copy_result_from_city(pcity, &result);
+  gtk_label_set_text(GTK_LABEL(pdialog->result_label),
+		     cmafec_get_result_descr(pcity, &result, &param));
 
   /* if called from a hscale, we _don't_ want to do this */
   if (refresh != DONT_REFRESH_HSCALES) {
     set_hscales(&param, pdialog);
   }
 
-  if (refresh != DONT_REFRESH_SELECT) {
-    /* highlight preset if parameter matches */
-    preset_index = cmafec_preset_get_index_of_parameter(&param);
-    if (preset_index != -1) {
-      allow_refreshes = 0;
-      gtk_clist_select_row(GTK_CLIST(pdialog->preset_list), preset_index, 0);
-      allow_refreshes = 1;
-    } else {
-      gtk_clist_unselect_all(GTK_CLIST(pdialog->preset_list));
-    }
-  }
+  gtk_widget_queue_draw(pdialog->preset_list);
 
-  gtk_widget_set_sensitive(GTK_WIDGET(pdialog->change_command),
-			   result.found_a_valid && !controlled);
-  gtk_widget_set_sensitive(GTK_WIDGET(pdialog->perm_command),
-			   result.found_a_valid && !controlled);
-  gtk_widget_set_sensitive(GTK_WIDGET(pdialog->release_command),
-			   controlled);
+  gtk_widget_set_sensitive(pdialog->active_command, can_client_issue_orders());
+
+  g_signal_handlers_disconnect_matched(pdialog->active_command,
+      G_SIGNAL_MATCH_FUNC,
+      0, 0, NULL, cma_active_callback, NULL);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdialog->active_command),
+      controlled);
+  g_signal_connect(pdialog->active_command, "clicked",
+      G_CALLBACK(cma_active_callback), pdialog);
+
+  if (controlled) {
+    gtk_image_set_from_stock(GTK_IMAGE(pdialog->active_image),
+	GTK_STOCK_YES, GTK_ICON_SIZE_DND);
+    gtk_label_set_text_with_mnemonic(GTK_LABEL(pdialog->active_label),
+	_("CMA Enabl_ed"));
+  } else {
+    gtk_image_set_from_stock(GTK_IMAGE(pdialog->active_image),
+	GTK_STOCK_NO, GTK_ICON_SIZE_DND);
+    gtk_label_set_text_with_mnemonic(GTK_LABEL(pdialog->active_label),
+	_("CMA Disabl_ed"));
+  }
+  gtk_widget_set_sensitive(pdialog->result_label, controlled);
 }
 
 /**************************************************************************
@@ -361,86 +472,49 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
 **************************************************************************/
 static void update_cma_preset_list(struct cma_dialog *pdialog)
 {
+  char buf[BUFFER_SIZE];
+  GtkTreeIter it;
   int i;
-  char *row[1];
-  char buf[1][BUFFER_SIZE];
-
-  row[0] = buf[0];
 
   /* Fill preset list */
-  gtk_clist_freeze(GTK_CLIST(pdialog->preset_list));
-  gtk_clist_clear(GTK_CLIST(pdialog->preset_list));
+  gtk_list_store_clear(pdialog->store);
 
-  /* if there are no presets preset, print informational message else 
-   * append the presets */
+  /* Append the presets */
   if (cmafec_preset_num()) {
+    gtk_tooltips_disable(pdialog->tips);
+
     for (i = 0; i < cmafec_preset_num(); i++) {
-      mystrlcpy(buf[0], cmafec_preset_get_descr(i), BUFFER_SIZE);
-      gtk_clist_insert(GTK_CLIST(pdialog->preset_list), i, row);
+      mystrlcpy(buf, cmafec_preset_get_descr(i), sizeof(buf));
+      gtk_list_store_append(pdialog->store, &it);
+      gtk_list_store_set(pdialog->store, &it, 0, buf, -1);
     }
   } else {
-    static char *info_message[4] = {
-      N_("For information on:"),
-      N_("CMA and presets"),
-      N_("including sample presets,"),
-      N_("see README.cma.")
-    };
-    static bool info_message_done;
-
-    intl_slist(ARRAY_SIZE(info_message), info_message, &info_message_done);
-
-    for (i = 0; i < 4; i++) {
-      mystrlcpy(buf[0], info_message[i], BUFFER_SIZE);
-      gtk_clist_insert(GTK_CLIST(pdialog->preset_list), i, row);
-      gtk_clist_set_selectable(GTK_CLIST(pdialog->preset_list), i, FALSE);
-    }
+    gtk_tooltips_enable(pdialog->tips);
   }
-  gtk_clist_thaw(GTK_CLIST(pdialog->preset_list));
 }
 
 /****************************************************************
- callback for selecting a preset from the preset clist
+ callback for selecting a preset from the preset view
 *****************************************************************/
-static void cma_select_preset_callback(GtkWidget *w, gint row, gint col,
-				       GdkEvent *ev, gpointer data)
+static void cma_activate_preset_callback(GtkTreeView *view, GtkTreePath *path,
+				         GtkTreeViewColumn *col, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  GList *selection = GTK_CLIST(pdialog->preset_list)->selection;
   int preset_index;
-  const struct cma_parameter *pparam;
+  const struct cm_parameter *pparam;
 
-  if (!selection) {
-    return;
-  }
-
-  preset_index = GPOINTER_TO_INT(selection->data);
-
-  if (ev && ev->type == GDK_2BUTTON_PRESS) {
-    /* Double-click to remove preset from list */
-    cma_preset_remove(pdialog, preset_index);
-    return;
-  }
+  preset_index = gtk_tree_path_get_indices(path) [0];
 
   pparam = cmafec_preset_get_parameter(preset_index);
 
   /* save the change */
   cmafec_set_fe_parameter(pdialog->pcity, pparam);
 
-  if (allow_refreshes) {
-    if (cma_is_city_under_agent(pdialog->pcity, NULL)) {
-      cma_release_city(pdialog->pcity);
-      cma_put_city_under_agent(pdialog->pcity, pparam);
-
-      /* unfog the city map if we were unable to put back under */
-      if (!cma_is_city_under_agent(pdialog->pcity, NULL)) {
-	refresh_city_dialog(pdialog->pcity);
-	return;			/* refreshing the city, refreshes cma */
-      } else {
-        city_report_dialog_update_city(pdialog->pcity);
-      }
-    }
-    refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_SELECT);
+  if (cma_is_city_under_agent(pdialog->pcity, NULL)) {
+    cma_release_city(pdialog->pcity);
+    cma_put_city_under_agent(pdialog->pcity, pparam);
   }
+  refresh_city_dialog(pdialog->pcity);
 }
 
 /**************************************************************************
@@ -449,12 +523,12 @@ static void cma_select_preset_callback(GtkWidget *w, gint row, gint col,
 static void cma_add_preset_callback(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  GList *selection = GTK_CLIST(pdialog->preset_list)->selection;
-  char *default_name;
+  const char *default_name;
   GtkWidget *parent = gtk_widget_get_toplevel(pdialog->shell);
+  int index;
 
-  if (selection) {
-    default_name = cmafec_preset_get_descr(GPOINTER_TO_INT(selection->data));
+  if ((index = gtk_tree_selection_get_row(pdialog->selection)) != -1) {
+    default_name = cmafec_preset_get_descr(index);
   } else {
     default_name = _("new preset");
   }
@@ -464,24 +538,22 @@ static void cma_add_preset_callback(GtkWidget *w, gpointer data)
 				    _("What should we name the preset?"),
 				    default_name,
 				    G_CALLBACK(cma_preset_add_callback_yes),
-				    (gpointer) pdialog,
+				    pdialog,
 				    G_CALLBACK(cma_preset_add_callback_no),
-				    (gpointer) pdialog);
+				    pdialog);
 
-  gtk_signal_connect(GTK_OBJECT(pdialog->name_shell), "delete_event",
-		     GTK_SIGNAL_FUNC(cma_preset_add_callback_delete), data);
+  g_signal_connect(pdialog->name_shell, "destroy",
+		   G_CALLBACK(cma_preset_add_callback_destroy), data);
 }
 
 /****************************************************************
  callback for the add_preset popup (delete popup)
 *****************************************************************/
-static int cma_preset_add_callback_delete(GtkWidget *w, GdkEvent *ev, 
-                                          gpointer data)
+static void cma_preset_add_callback_destroy(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
 
   pdialog->name_shell = NULL;
-  return FALSE;
 }
 
 /****************************************************************
@@ -489,12 +561,6 @@ static int cma_preset_add_callback_delete(GtkWidget *w, GdkEvent *ev,
 *****************************************************************/
 static void cma_preset_add_callback_no(GtkWidget *w, gpointer data)
 {
-  struct cma_dialog *pdialog = (struct cma_dialog *) data;
-
-  if (pdialog) {
-    pdialog->name_shell = NULL;
-  }
-
   input_dialog_destroy(w);
 }
 
@@ -506,7 +572,7 @@ static void cma_preset_add_callback_yes(GtkWidget *w, gpointer data)
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
 
   if (pdialog) {
-    struct cma_parameter param;
+    struct cm_parameter param;
 
     cmafec_get_fe_parameter(pdialog->pcity, &param);
     cmafec_preset_add(input_dialog_get_input(w), &param);
@@ -514,7 +580,6 @@ static void cma_preset_add_callback_yes(GtkWidget *w, gpointer data)
     refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_HSCALES);
     /* if this or other cities have this set as "custom" */
     city_report_dialog_update();
-    pdialog->name_shell = NULL;
   }
 
   input_dialog_destroy(w);
@@ -527,16 +592,19 @@ static gboolean cma_preset_key_pressed_callback(GtkWidget *w, GdkEventKey *ev,
 						gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  GList *selection = GTK_CLIST(pdialog->preset_list)->selection;
+  int index;
 
-  if (!selection) {
+  if ((index = gtk_tree_selection_get_row(pdialog->selection)) == -1) {
     return FALSE;
   }
 
   if (ev->type == GDK_KEY_PRESS) {
     switch (ev->keyval) {
     case GDK_Delete:
-      cma_preset_remove(pdialog, GPOINTER_TO_INT(selection->data));
+      cma_preset_remove(pdialog, index);
+      break;
+    case GDK_Insert:
+      cma_add_preset_callback(NULL, pdialog);
       break;
     default:
       return FALSE;
@@ -553,13 +621,13 @@ static gboolean cma_preset_key_pressed_callback(GtkWidget *w, GdkEventKey *ev,
 static void cma_del_preset_callback(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  GList *selection = GTK_CLIST(pdialog->preset_list)->selection;
+  int index;
 
-  if (!selection) {
+  if ((index = gtk_tree_selection_get_row(pdialog->selection)) == -1) {
     return;
   }
 
-  cma_preset_remove(pdialog, GPOINTER_TO_INT(selection->data));
+  cma_preset_remove(pdialog, index);
 }
 
 /**************************************************************************
@@ -570,15 +638,15 @@ static void cma_preset_remove(struct cma_dialog *pdialog, int preset_index)
   GtkWidget *parent = gtk_widget_get_toplevel(pdialog->shell), *shl;
 
   pdialog->id = preset_index;
-  shl = gtk_message_dialog_new(GTK_WINDOW(parent),
+  shl = gtk_message_dialog_new(NULL,
 			       GTK_DIALOG_DESTROY_WITH_PARENT,
 			       GTK_MESSAGE_QUESTION,
 			       GTK_BUTTONS_YES_NO,
 			       _("Remove this preset?"));
+  setup_dialog(shl, parent);
   pdialog->preset_remove_shell = shl;
 
-  gtk_window_set_title(GTK_WINDOW(shl),
-		       cmafec_preset_get_descr(preset_index));
+  gtk_window_set_title(GTK_WINDOW(shl), cmafec_preset_get_descr(preset_index));
   gtk_window_set_position(GTK_WINDOW(shl), GTK_WIN_POS_CENTER_ON_PARENT);
 
   g_signal_connect(shl, "response",
@@ -590,7 +658,7 @@ static void cma_preset_remove(struct cma_dialog *pdialog, int preset_index)
 /****************************************************************
  callback for the remove_preset popup
 *****************************************************************/
-static void cma_preset_remove_response(GtkWidget * w, gint response,
+static void cma_preset_remove_response(GtkWidget *w, gint response,
 				       gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
@@ -604,46 +672,25 @@ static void cma_preset_remove_response(GtkWidget * w, gint response,
     city_report_dialog_update();
   }
   gtk_widget_destroy(w);
-
+  
   pdialog->preset_remove_shell = NULL;
 }
 
 /**************************************************************************
- changes the workers of the city to the cma parameters
+ activates/deactivates agent control.
 **************************************************************************/
-static void cma_change_to_callback(GtkWidget *w, gpointer data)
-{
-  struct cma_result result;
-  struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  struct cma_parameter param;
-
-  cmafec_get_fe_parameter(pdialog->pcity, &param);
-  cma_query_result(pdialog->pcity, &param, &result);
-  cma_apply_result(pdialog->pcity, &result);
-}
-
-/**************************************************************************
- changes the workers of the city to the cma parameters and puts the
- city under agent control
-**************************************************************************/
-static void cma_change_permanent_to_callback(GtkWidget *w, gpointer data)
-{
-  struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  struct cma_parameter param;
-
-  cmafec_get_fe_parameter(pdialog->pcity, &param);
-  cma_put_city_under_agent(pdialog->pcity, &param);
-  refresh_city_dialog(pdialog->pcity);
-}
-
-/**************************************************************************
- releases the city from agent control
-**************************************************************************/
-static void cma_release_callback(GtkWidget *w, gpointer data)
+static void cma_active_callback(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
 
-  cma_release_city(pdialog->pcity);
+  if (cma_is_city_under_agent(pdialog->pcity, NULL)) {
+    cma_release_city(pdialog->pcity);
+  } else {
+    struct cm_parameter param;
+
+    cmafec_get_fe_parameter(pdialog->pcity, &param);
+    cma_put_city_under_agent(pdialog->pcity, &param);
+  }
   refresh_city_dialog(pdialog->pcity);
 }
 
@@ -651,7 +698,7 @@ static void cma_release_callback(GtkWidget *w, gpointer data)
  called to adjust the sliders when a preset is selected
  notice that we don't want to call update_result here. 
 *****************************************************************/
-static void set_hscales(const struct cma_parameter *const parameter,
+static void set_hscales(const struct cm_parameter *const parameter,
 			struct cma_dialog *pdialog)
 {
   int i;
@@ -675,7 +722,7 @@ static void set_hscales(const struct cma_parameter *const parameter,
 static void hscale_changed(GtkAdjustment *get, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  struct cma_parameter param;
+  struct cm_parameter param;
   int i;
 
   if (!allow_refreshes) {
@@ -687,7 +734,6 @@ static void hscale_changed(GtkAdjustment *get, gpointer data)
     param.minimal_surplus[i] = (int) (pdialog->minimal_surplus[i]->value);
     param.factor[i] = (int) (pdialog->factor[i]->value);
   }
-  param.factor_target = FT_SURPLUS;
   param.require_happy =
       (GTK_TOGGLE_BUTTON(pdialog->happy_button)->active ? 1 : 0);
   param.happy_factor = (int) (pdialog->factor[NUM_STATS]->value);
@@ -699,15 +745,9 @@ static void hscale_changed(GtkAdjustment *get, gpointer data)
   if (cma_is_city_under_agent(pdialog->pcity, NULL)) {
     cma_release_city(pdialog->pcity);
     cma_put_city_under_agent(pdialog->pcity, &param);
-
-    /* unfog the city map if we were unable to put back under */
-    if (!cma_is_city_under_agent(pdialog->pcity, NULL)) {
-      refresh_city_dialog(pdialog->pcity);
-      return;			/* refreshing city refreshes cma */
-    } else {
-      city_report_dialog_update_city(pdialog->pcity);
-    }
+    refresh_city_dialog(pdialog->pcity);
+  } else {
+    refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_HSCALES);
   }
-
-  refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_HSCALES);
 }
+
