@@ -10,34 +10,38 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 
-#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 
 #include "city.h"
 #include "fcintl.h"
 #include "game.h"
+#include "mem.h"
+#include "packets.h"
+#include "support.h"
+#include "worklist.h"
+
+#include "chatline_common.h"
+#include "civclient.h"
+#include "climisc.h"
+#include "clinet.h"
+
+#include "citydlg.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdlg.h"
 #include "inputdlg.h"
-#include "mem.h"
-#include "packets.h"
-#include "worklist.h"
-#include "support.h"
-#include "log.h"
-#include "climisc.h"
-#include "clinet.h"
 
 #include "wldlg.h"
-#include "citydlg.h"
 
 #define WORKLIST_ADVANCED_TARGETS  	1
 #define WORKLIST_CURRENT_TARGETS   	0
@@ -139,7 +143,7 @@ void popup_worklists_report(struct player *pplr)
 {
   GtkWidget *button, *scrolled;
   GtkAccelGroup *accel;
-  char *title[1] = { N_("Available worklists") };
+  const char *title[1] = { N_("Available worklists") };
   static char **clist_title = NULL;
 
   /* Report window already open */
@@ -307,13 +311,13 @@ struct worklist_editor *create_worklist_editor(struct worklist *pwl,
   GtkAccelGroup *accel = gtk_accel_group_new();
 
   static char **wl_clist_titles = NULL;
-  char *wl_titles[] = { N_("Type"),
+  const char *wl_titles[] = { N_("Type"),
     N_("Info"),
     N_("Cost")
   };
 
   static char **avail_clist_titles = NULL;
-  char *avail_titles[] = { N_("Type"),
+  const char *avail_titles[] = { N_("Type"),
     N_("Info"),
     N_("Cost"),
     N_("Turns")
@@ -910,7 +914,7 @@ static void worklist_select_callback(GtkWidget * w, gint row, gint column,
   int row_selected;
   struct worklist_editor *peditor = (struct worklist_editor *) data;
 
-  if (ev && ev->type == GDK_2BUTTON_PRESS) {
+  if (can_client_issue_orders() && ev && ev->type == GDK_2BUTTON_PRESS) {
     /* Double-click to remove item from worklist */
     worklist_remove_item(peditor);
     return;
@@ -918,8 +922,10 @@ static void worklist_select_callback(GtkWidget * w, gint row, gint column,
 
   row_selected = (GTK_CLIST(peditor->worklist)->selection != NULL);
 
-  gtk_widget_set_sensitive(peditor->btn_up, row_selected && row > 0);
-  gtk_widget_set_sensitive(peditor->btn_down, row_selected &&
+  gtk_widget_set_sensitive(peditor->btn_up, can_client_issue_orders() &&
+			   row_selected && row > 0 );
+  gtk_widget_set_sensitive(peditor->btn_down, can_client_issue_orders() &&
+			   row_selected &&
 			   row < GTK_CLIST(peditor->worklist)->rows - 1);
 }
 
@@ -929,7 +935,7 @@ static void worklist_select_callback(GtkWidget * w, gint row, gint column,
 static void targets_select_callback(GtkWidget * w, gint row, gint column,
 				    GdkEvent * ev, gpointer data)
 {
-  if (ev && ev->type == GDK_2BUTTON_PRESS) {
+  if (can_client_issue_orders() && ev && ev->type == GDK_2BUTTON_PRESS) {
     struct worklist_editor *peditor = (struct worklist_editor *) data;
     /* Double-click to insert item in worklist */
     worklist_insert_item(peditor);
@@ -992,10 +998,15 @@ static void worklist_insert_item(struct worklist_editor *peditor)
   int where, index, len;
   wid wid;
 
-  if (!listSelection)
+  if (!listSelection) {
     where = MAX_LEN_WORKLIST;
-  else
+  } else {
     where = GPOINTER_TO_INT(listSelection->data);
+    if(peditor->pcity->did_buy && where == 0) {
+      append_output_window(_("Game: You have bought this turn, can't change."));
+      where = 1;
+    }
+  }
 
   /* Is there anything selected to insert? */
   if (!availSelection) {
@@ -1164,6 +1175,12 @@ static void worklist_swap_up_callback(GtkWidget * w, gpointer data)
   if (idx == 0)
     return;
 
+  if(idx == 1 && peditor->pcity->did_buy) {
+    /* Refuse to swap the first item if we've bought. */
+    append_output_window(_("Game: You have bought this turn, can't change."));
+    return;
+  }
+
   worklist_swap_entries(idx, idx - 1, peditor);
   worklist_list_update(peditor);
   gtk_clist_select_row(GTK_CLIST(peditor->worklist), idx - 1, 0);
@@ -1185,6 +1202,12 @@ static void worklist_swap_down_callback(GtkWidget * w, gpointer data)
 
   idx = GPOINTER_TO_INT(selection->data);
 
+  if(idx == 0 && peditor->pcity->did_buy) {
+    /* Refuse to swap the first item if we've bought. */
+    append_output_window(_("Game: You have bought this turn, can't change."));
+    return;
+  }
+
   if (idx == MAX_LEN_WORKLIST - 1 ||
       peditor->worklist_wids[idx + 1] == WORKLIST_END) return;
 
@@ -1440,6 +1463,14 @@ static void cleanup_worklist_editor(struct worklist_editor *peditor)
   free(peditor);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+void close_worklist_editor(struct worklist_editor *peditor)
+{
+  cleanup_worklist_editor(peditor);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
