@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -35,6 +36,7 @@
 #include "map.h"
 #include "options.h"
 
+#include "climap.h"
 #include "climisc.h"
 #include "colors.h"
 #include "control.h"
@@ -66,9 +68,9 @@ void free_sprite(struct Sprite *sprite)
 /****************************************************************
  Return the gfx file extension the client supports
 *****************************************************************/
-char **gfx_fileextensions(void)
+const char **gfx_fileextensions(void)
 {
-  static char *ext[] =
+  static const char *ext[] =
   {
     "png",
     "ilbm",
@@ -387,14 +389,6 @@ int render_sprites(APTR drawhandle)
   return TRUE;
 }
 
-/****************************************************************
- Returns a cititen sprite
-*****************************************************************/
-struct Sprite *get_citizen_sprite(int frame)
-{
-  return sprites.citizen[CLIP(0, frame, NUM_TILES_CITIZEN - 1)];
-}
-
 /****************************************************************
  Returns a thumb sprite
 *****************************************************************/
@@ -875,8 +869,7 @@ void put_unit_tile(struct RastPort *rp, struct unit *punit, int x1, int y1)
     }
     else
     {
-      if (flags_are_transparent) put_sprite_overlay(rp, sprites[0], x1, y1);
-      else put_sprite(rp, sprites[0], x1, y1);
+      put_sprite_overlay(rp, sprites[0], x1, y1);
     }
 
     for (i = 1; i < count; i++)
@@ -949,14 +942,13 @@ void put_tile(struct RastPort *rp, int x, int y, int canvas_x, int canvas_y, int
     }
 
     if (draw_coastline && !draw_terrain) {
-      enum tile_terrain_type t1 = map_get_terrain(x, y), t2;
+      Terrain_type_id t1 = map_get_terrain(x, y), t2;
       int x1 = x-1, y1 = y;
       SetAPen(rp,GetColorPen(COLOR_STD_OCEAN));
       if (normalize_map_pos(&x1, &y1)) {
 	t2 = map_get_terrain(x1, y1);
 	/* left side */
-	if ((t1 == T_OCEAN) ^ (t2 == T_OCEAN))
-	{
+	if (is_ocean(t1) ^ is_ocean(t2)) {
 	  Move(rp, canvas_x, canvas_y);
 	  Draw(rp, canvas_x, canvas_y + NORMAL_TILE_HEIGHT-1);
 	}
@@ -965,8 +957,7 @@ void put_tile(struct RastPort *rp, int x, int y, int canvas_x, int canvas_y, int
       x1 = x; y1 = y-1;
       if (normalize_map_pos(&x1, &y1)) {
 	t2 = map_get_terrain(x1, y1);
-	if ((t1 == T_OCEAN) ^ (t2 == T_OCEAN))
-	{
+	if (is_ocean(t1) ^ is_ocean(t2)) {
 	  Move(rp, canvas_x, canvas_y);
 	  Draw(rp, canvas_x + NORMAL_TILE_WIDTH-1, canvas_y);
 	}
@@ -979,10 +970,10 @@ void put_tile(struct RastPort *rp, int x, int y, int canvas_x, int canvas_y, int
 
   if (!citymode) {
     /* put any goto lines on the tile. */
-    if (is_real_tile(x, y)) {
+    if (is_real_map_pos(x, y)) {
       int dir;
       for (dir = 0; dir < 8; dir++) {
-	if (get_drawn(x, y, dir)) {
+	if (is_drawn_line(x, y, dir)) {
 	  put_line(rp, 0,0,x, y, dir);
 	}
       }
@@ -993,7 +984,7 @@ void put_tile(struct RastPort *rp, int x, int y, int canvas_x, int canvas_y, int
       int line_x = x - 1;
       int line_y = y;
       if (normalize_map_pos(&line_x, &line_y)
-	  && get_drawn(line_x, line_y, 2)) {
+	  && is_drawn_line(line_x, line_y, 2)) {
 	/* it is really only one pixel in the top right corner */
 	put_line(rp, 0,0,line_x, line_y, 2);
       }
@@ -1248,7 +1239,7 @@ static void put_tile_iso(struct RastPort *rp, int x, int y,
     return;
   }
 
-  assert(is_real_tile(x, y));
+  assert(is_real_map_pos(x, y));
   normalize_map_pos(&x, &y);
   fog = tile_get_known(x, y) == TILE_KNOWN_FOGGED && draw_fog_of_war;
   pcity = map_get_city(x, y);
@@ -1276,7 +1267,7 @@ static void put_tile_iso(struct RastPort *rp, int x, int y,
   }
 
   if (draw_terrain) {
-    if (map_get_terrain(x, y) == T_OCEAN) { /* coasts */
+    if (is_ocean(map_get_terrain(x, y))) { /* coasts */
       int dx, dy;
 
       /* top */
@@ -1352,13 +1343,13 @@ static void put_tile_iso(struct RastPort *rp, int x, int y,
   }
 
   if (draw_coastline && !draw_terrain) {
-    enum tile_terrain_type t1 = map_get_terrain(x, y), t2;
+    Terrain_type_id t1 = map_get_terrain(x, y), t2;
     int x1, y1;
     SetAPen(rp,GetColorPen(COLOR_STD_OCEAN));
     x1 = x; y1 = y-1;
     if (normalize_map_pos(&x1, &y1)) {
       t2 = map_get_terrain(x1, y1);
-      if (draw & D_M_R && ((t1 == T_OCEAN) ^ (t2 == T_OCEAN))) {
+      if (draw & D_M_R && (is_ocean(t1) ^ is_ocean(t2))) {
       	Move(rp, canvas_x + NORMAL_TILE_WIDTH/2, canvas_y);
       	Draw(rp, canvas_x + NORMAL_TILE_WIDTH, canvas_y+NORMAL_TILE_HEIGHT/2);
       }
@@ -1366,7 +1357,7 @@ static void put_tile_iso(struct RastPort *rp, int x, int y,
     x1 = x-1; y1 = y;
     if (normalize_map_pos(&x1, &y1)) {
       t2 = map_get_terrain(x1, y1);
-      if (draw & D_M_L && ((t1 == T_OCEAN) ^ (t2 == T_OCEAN))) {
+      if (draw & D_M_L && (is_ocean(t1) ^ is_ocean(t2))) {
       	Move(rp, canvas_x, canvas_y + NORMAL_TILE_HEIGHT/2);
       	Draw(rp, canvas_x+NORMAL_TILE_WIDTH/2, canvas_y);
       }
