@@ -10,15 +10,14 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/  
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif        
-#include <stdlib.h>
- 
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
                          
 #include <windows.h>
 #include <windowsx.h>
@@ -42,6 +41,7 @@
 #include "options.h"
 #include "repodlgs.h"
 #include "climisc.h"
+#include "text.h"
                            
 #include "cityrep.h"
 extern HINSTANCE freecivhinst;
@@ -127,7 +127,7 @@ static void create_cityrep_config_dlg(HWND hWnd)
   box=fcwin_vbox_new(hWnd,FALSE);
   fcwin_box_add_static_default(box,_("Set columns shown"),-1,SS_CENTER);
   for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {     
-    fcwin_box_add_checkbox(box,_(spec->explanation),
+    fcwin_box_add_checkbox(box, spec->explanation,
 			   ID_CITYREP_CONFIG_BASE+i,0,FALSE,FALSE,5);
   }
   fcwin_box_add_button(box,_("Close"),IDOK,0,FALSE,FALSE,15);
@@ -188,24 +188,29 @@ append_impr_or_unit_to_menu_sub(HMENU menu,
 {
   cid cids[U_LAST + B_LAST];
   struct item items[U_LAST + B_LAST];
-  int item, cids_used, num_selected_cities=0;
-  struct city *selected_cities[200];
+  int item, cids_used, num_selected_cities = 0;
+  struct city **selected_cities = NULL;
 
   if (change_prod) {
     int j;
     HWND hList = GetDlgItem(hCityRep, ID_CITYREP_LIST);
 
-    for (j = 0; j < selcount; j++) {
-      selected_cities[j] = (struct city *) ListBox_GetItemData(hList,
-							       selitems
-							       [j]);
-    }
     num_selected_cities = selcount;
+    selected_cities =
+	fc_malloc(sizeof(*selected_cities) * num_selected_cities);
+    
+    for (j = 0; j < num_selected_cities; j++) {
+      selected_cities[j] = (struct city *) ListBox_GetItemData(hList,
+							       selitems[j]);
+    }
   }
 
   cids_used = collect_cids1(cids, selected_cities,
 			    num_selected_cities, append_units,
 			    append_wonders, change_prod, test_func);
+  if (selected_cities) {
+    free(selected_cities);
+  }
   name_and_sort_items(cids, cids_used, items, change_prod, NULL);
 
   for (item = 0; item < cids_used; item++) {
@@ -279,8 +284,8 @@ static void get_city_table_header(char *text[], int n)
   int i;
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
     my_snprintf(text[i], n, "%*s\n%*s",
-            NEG_VAL(spec->width), spec->title1 ? _(spec->title1): "",
-            NEG_VAL(spec->width), spec->title2 ? _(spec->title2): "");
+            NEG_VAL(spec->width), spec->title1 ? spec->title1: "",
+            NEG_VAL(spec->width), spec->title2 ? spec->title2: "");
   }
 }             
                   
@@ -314,7 +319,7 @@ static void cityrep_center(HWND hWnd)
     return;
   pcity=(struct city *)ListBox_GetItemData(GetDlgItem(hWnd,ID_CITYREP_LIST),
 					   id);
-  center_tile_mapcanvas(pcity->x,pcity->y);
+  center_tile_mapcanvas(pcity->tile);
 }
 
 /**************************************************************************
@@ -342,52 +347,30 @@ static void cityrep_popup(HWND hWnd)
 /**************************************************************************
 
 **************************************************************************/
-static void cityrep_buy(HWND hWnd)
+static void cityrep_do_buy(HWND hWnd)
 {
   int cityids[256];
-  int selcount;
   int i;
-  struct city *pcity;
-   selcount=ListBox_GetSelCount(GetDlgItem(hWnd,ID_CITYREP_LIST));
-  if (selcount==LB_ERR) return;
-  selcount=MIN(256,selcount);
-  selcount=ListBox_GetSelItems(GetDlgItem(hWnd,ID_CITYREP_LIST),
-			       selcount,&cityids[0]);
-  for(i=0;i<selcount;i++)
-    {
-      int value;
-      char *name;
-      char buf[512];            
-      pcity=(struct city *)ListBox_GetItemData(GetDlgItem(hWnd,
-							  ID_CITYREP_LIST),
-					       cityids[i]);
-      value=city_buy_cost(pcity);
-      if(pcity->is_building_unit)
-        name=get_unit_type(pcity->currently_building)->name;
-      else
-        name=get_impr_name_ex(pcity, pcity->currently_building);
- 
-      if (game.player_ptr->economic.gold >= value)
-        {
-          struct packet_city_request packet;
-          packet.city_id=pcity->id;
-          send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
-        }
-      else
-        {
-          my_snprintf(buf, sizeof(buf),
-                      _("Game: %s costs %d gold and you only have %d gold."),
-                      name,value,game.player_ptr->economic.gold);
-          append_output_window(buf);
-        }             
-    }
+  int selcount = ListBox_GetSelCount(GetDlgItem(hWnd, ID_CITYREP_LIST));
+
+  if (selcount == LB_ERR) {
+    return;
+  }
+  selcount = MIN(256, selcount);
+  selcount = ListBox_GetSelItems(GetDlgItem(hWnd, ID_CITYREP_LIST),
+				 selcount, &cityids[0]);
+  for (i = 0; i < selcount; i++) {
+    cityrep_buy((struct city *) ListBox_GetItemData(GetDlgItem(hWnd,
+							       ID_CITYREP_LIST),
+						    cityids[i]));
+  }
 }
   
-
+#if 0
 /**************************************************************************
 
 **************************************************************************/
-void cityrep_refresh(HWND hWnd)
+static void cityrep_refresh(HWND hWnd)
 {
     int cityids[256];
   int selcount;
@@ -414,6 +397,8 @@ void cityrep_refresh(HWND hWnd)
 				  &packet);      
     }
 }
+#endif
+
 /**************************************************************************
 
 **************************************************************************/
@@ -486,7 +471,7 @@ static void cityrep_select(HWND hWnd)
   AppendMenu(popup,MF_POPUP,(UINT)submenu,_("Improvements in City"));
   max_improvement_id=ID_IMPROVEMENTS_POPUP_BASE;
   append_impr_or_unit_to_menu(submenu, FALSE, TRUE, FALSE,
-			      (TestCityFunc*)city_got_building,
+			      city_building_present,
 			      &cityids,selcount,
 			      &max_improvement_id);
   
@@ -514,15 +499,11 @@ static void cityrep_change_menu(HWND hWnd, cid cid)
 
   connection_do_buffer(&aconnection);
   for (i = 0; i < selcount; i++) {
-    struct packet_city_request packet;
     pcity = (struct city *) ListBox_GetItemData(GetDlgItem(hWnd,
 							   ID_CITYREP_LIST),
 						cityids[i]);
-    packet.city_id = pcity->id;
-    packet.build_id = number;
-    packet.is_build_id_unit_id = is_unit;
     last_request_id =
-	send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
+      city_change_production(pcity, is_unit, number);
     ListBox_SetSel(GetDlgItem(hWnd, ID_CITYREP_LIST), FALSE, cityids[i]);
   }
 
@@ -570,8 +551,9 @@ static void list_coastal_select(HWND hLst)
   for (i=0;i<num;i++)
     {
       pcity=(struct city *)ListBox_GetItemData(hLst,i);
-      if (is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN)) 
+      if (is_ocean_near_tile(pcity->tile)) {
 	ListBox_SetSel(hLst,TRUE,i);
+      }
     }
 }
 
@@ -598,8 +580,8 @@ static void list_sameisland_select(HWND hLst)
 	{
 	  struct city *selectedcity;
 	  selectedcity=(struct city *)ListBox_GetItemData(hLst,cityids[j]);
-          if (map_get_continent(pcity->x, pcity->y)
-              == map_get_continent(selectedcity->x, selectedcity->y))
+          if (map_get_continent(pcity->tile)
+              == map_get_continent(selectedcity->tile))
 	    {    
 	      ListBox_SetSel(hLst,TRUE,i);
 	      break;
@@ -611,7 +593,8 @@ static void list_sameisland_select(HWND hLst)
 /**************************************************************************
 
 **************************************************************************/
-void list_impr_or_unit_select(HWND hLst,int num, TestCityFunc *test_func)
+static void list_impr_or_unit_select(HWND hLst, int num,
+				     TestCityFunc *test_func)
 {
   int i,rows;
   list_all_select(hLst,FALSE);
@@ -658,7 +641,7 @@ static void menu_proc(HWND hWnd,int cmd, DWORD num)
   if ((cmd>=ID_IMPROVEMENTS_POPUP_BASE)&&
       (cmd<max_improvement_id))
     {
-      list_impr_or_unit_select(hLst,num,city_got_building);
+      list_impr_or_unit_select(hLst,num,city_building_present);
       max_improvement_id=0;
     }
 }
@@ -827,8 +810,8 @@ static void list_minsize(LPPOINT minsize,void *data)
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {    
     if (!spec->show) continue;
     width=spec->width>0?spec->width:-spec->width;
-    width=MAX(strlen(spec->title1 ? _(spec->title1): ""),width);
-    width=MAX(strlen(spec->title2 ? _(spec->title2): ""),width);
+    width = MAX(strlen(spec->title1 ? spec->title1: ""), width);
+    width = MAX(strlen(spec->title2 ? spec->title2: ""), width);
     width_total+=width+2;
   }
   minsize->x=(list_singlechar.right-list_singlechar.left)*width_total;
@@ -849,8 +832,8 @@ static void list_setsize(LPRECT setsize,void *data)
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
     if (!sort_buttons[i]) continue;
     w=spec->width>0?spec->width:-spec->width;
-    w=MAX(w,strlen(spec->title1 ? _(spec->title1): ""));
-    w=MAX(w,strlen(spec->title2 ? _(spec->title2): ""));
+    w = MAX(strlen(spec->title1 ? spec->title1: ""), w);
+    w = MAX(strlen(spec->title2 ? spec->title2: ""), w);
     w+=2;
     w*=(list_singlechar.right-list_singlechar.left);
     MoveWindow(sort_buttons[i],x,y,w-1,button_h,TRUE);
@@ -1012,7 +995,7 @@ static LONG APIENTRY city_report_proc(HWND hWnd,
 	  cityrep_popup(hWnd);
 	  break;
 	case ID_CITYREP_BUY:
-	  cityrep_buy(hWnd);
+	  cityrep_do_buy(hWnd);
 	  break;
 	case ID_CITYREP_CHANGE:
 	  cityrep_change(hWnd);
@@ -1147,13 +1130,12 @@ city_report_dialog_update(void)
   char  buf   [NUM_CREPORT_COLS][64];
   char full_row[1024];
   int   i;
-  char *report_title;
   struct city_report_spec *spec; 
   if(is_report_dialogs_frozen()) return;    
   if (!hCityRep)
     return;
-  report_title=get_report_title(_("City Advisor"));      
-  SetWindowText(GetDlgItem(hCityRep,ID_CITYREP_TOP),report_title);
+  SetWindowText(GetDlgItem(hCityRep, ID_CITYREP_TOP),
+		get_report_title(_("City Advisor")));
   for (i=0, spec=city_report_specs;i<NUM_CREPORT_COLS;i++, spec++)
     {
       row[i] = buf[i];    
@@ -1211,3 +1193,21 @@ city_report_dialog_update_city(struct city *pcity)
 	}
     }
 }
+
+/****************************************************************
+ After a selection rectangle is defined, make the cities that
+ are hilited on the canvas exclusively hilited in the
+ City List window.
+*****************************************************************/
+void hilite_cities_from_canvas(void)
+{
+  /* PORTME */
+}
+
+/****************************************************************
+ Toggle a city's hilited status.
+*****************************************************************/
+void toggle_city_hilite(struct city *pcity, bool on_off)
+{
+  /* PORTME */
+}
