@@ -10,14 +10,16 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
- 
+
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
+
 #include <windows.h>
 #include <windowsx.h>
  
@@ -49,7 +51,9 @@
 #define ID_HELP_IMPROVEMENT_LINK 112
 #define ID_HELP_WONDER_LINK 113
 
-extern char long_buffer[64000];
+/* HACK: we use a static string for convenience. */
+static char long_buffer[64000];
+
 extern HINSTANCE freecivhinst;
 extern HWND root_window;
 static HWND helpdlg_win;
@@ -218,8 +222,35 @@ static void set_help_text(char *txt)
 **************************************************************************/
 static void edit_minsize(POINT *pt,void *data)
 {
-  pt->x=300;
-  pt->y=100;
+  char buf[32768];
+  HWND hwnd = (HWND) data;
+  HFONT old;
+  HFONT font;
+  HDC hdc;
+  RECT rc;
+  old = NULL; /* just silence gcc */
+  buf[0] = 0;
+  GetWindowText(hwnd, buf, sizeof(buf));
+  if (strlen(buf)<10) {
+    pt->x = 300;
+    pt->y = 300;
+    return;
+  }
+  
+  hdc = GetDC(hwnd);
+  if ((font = GetWindowFont(hwnd))) {
+    old = SelectObject(hdc, font);
+  }
+  rc.left = 0;
+  rc.right = 10000;
+  rc.top=0;
+  DrawText(hdc, buf, strlen(buf), &rc, DT_CALCRECT);
+  pt->x = rc.right - rc.left + 40;
+  pt->y = 300;
+  if (font) {
+    SelectObject(hdc, old);
+  }
+  ReleaseDC(hwnd, hdc);
 }
 
 /**************************************************************************
@@ -439,13 +470,13 @@ void create_help_dialog()
 static void help_update_improvement(const struct help_item *pitem,
                                     char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
  
   create_help_page(HELP_IMPROVEMENT);
  
   if (which<B_LAST) {
     struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d", imp->build_cost);
+    sprintf(buf, "%d", impr_build_shield_cost(which));
     SetWindowText(help_ilabel[1], buf);
     sprintf(buf, "%d", imp->upkeep);
     SetWindowText(help_ilabel[3], buf);
@@ -462,7 +493,7 @@ static void help_update_improvement(const struct help_item *pitem,
     SetWindowText(help_ilabel[5], _("(Never)"));
 /*    create_tech_tree(help_improvement_tree, 0, game.num_tech_types, 3);*/
   }
-  helptext_improvement(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   set_help_text(buf);
 
 }
@@ -472,20 +503,25 @@ static void help_update_improvement(const struct help_item *pitem,
 static void help_update_wonder(const struct help_item *pitem,
                                char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
  
   create_help_page(HELP_WONDER);
  
   if (which<B_LAST) {
     struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d", imp->build_cost);
+    sprintf(buf, "%d", impr_build_shield_cost(which));
     SetWindowText(help_ilabel[1], buf);
     if (imp->tech_req == A_LAST) {
       SetWindowText(help_ilabel[3], _("(Never)"));
     } else {
       SetWindowText(help_ilabel[3], advances[imp->tech_req].name);
     }
-    SetWindowText(help_ilabel[5], advances[imp->obsolete_by].name);
+    if (tech_exists(imp->obsolete_by)) {
+      SetWindowText(help_ilabel[5], advances[imp->obsolete_by].name);
+    } else {
+      SetWindowText(help_ilabel[5], _("(Never)"));
+    }
+
     /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
@@ -496,7 +532,7 @@ static void help_update_wonder(const struct help_item *pitem,
 /*    create_tech_tree(help_improvement_tree, 0, game.num_tech_types, 3); */
   }
  
-  helptext_wonder(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   set_help_text(buf);
 }                            
 
@@ -510,116 +546,99 @@ static void help_update_terrain(const struct help_item *pitem,
   
   create_help_page(HELP_TERRAIN);
 
-  if (i < T_COUNT)
-    {
-      sprintf (buf, "%d/%d.%d",
-	       tile_types[i].movement_cost,
-	       (int)(tile_types[i].defense_bonus/10), tile_types[i].defense_bonus%10);
-      SetWindowText (help_tlabel[0][1], buf);
+  if (i < T_COUNT) {
+    sprintf(buf, "%d/%d.%d",
+	    tile_types[i].movement_cost,
+	    (int)(tile_types[i].defense_bonus/10),
+	    tile_types[i].defense_bonus%10);
+    SetWindowText (help_tlabel[0][1], buf);
 
-      sprintf (buf, "%d/%d/%d",
-	       tile_types[i].food,
-	       tile_types[i].shield,
-	       tile_types[i].trade);
-      SetWindowText (help_tlabel[0][4], buf);
+    sprintf(buf, "%d/%d/%d",
+	    tile_types[i].food,
+	    tile_types[i].shield,
+	    tile_types[i].trade);
+    SetWindowText(help_tlabel[0][4], buf);
 
-      if (*(tile_types[i].special_1_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_1_name);
-	  SetWindowText (help_tlabel[1][0], buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_1,
-		   tile_types[i].shield_special_1,
-		   tile_types[i].trade_special_1);
-	  SetWindowText (help_tlabel[1][1], buf);
-	} else {
-	  SetWindowText (help_tlabel[1][0], " ");
-	  SetWindowText (help_tlabel[1][1], " ");
-	}
-
-      if (*(tile_types[i].special_2_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_2_name);
-	  SetWindowText (help_tlabel[1][3], buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_2,
-		   tile_types[i].shield_special_2,
-		   tile_types[i].trade_special_2);
-	  SetWindowText (help_tlabel[1][4], buf);
-	} else {
-	  SetWindowText (help_tlabel[1][3], " ");
-	  SetWindowText (help_tlabel[1][4], " ");
-	}
-
-      if (tile_types[i].road_trade_incr > 0)
-	{
-	  sprintf (buf, _("+%d Trade / %d"),
-		   tile_types[i].road_trade_incr,
-		   tile_types[i].road_time);
-	}
-      else if (tile_types[i].road_time > 0)
-	{
-	  sprintf (buf, _("no extra / %d"),
-		   tile_types[i].road_time);
-	}
-      else
-	{
-	  strcpy (buf, _("n/a"));
-	}
-      SetWindowText (help_tlabel[2][1], buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].irrigation_result == i)
-	{
-	  if (tile_types[i].irrigation_food_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Food / %d"),
-		       tile_types[i].irrigation_food_incr,
-		       tile_types[i].irrigation_time);
-	    }
-	}
-      else if (tile_types[i].irrigation_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].irrigation_result].terrain_name,
-		   tile_types[i].irrigation_time);
-	}
-      SetWindowText (help_tlabel[2][4], buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].mining_result == i)
-	{
-	  if (tile_types[i].mining_shield_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Res. / %d"),
-		       tile_types[i].mining_shield_incr,
-		       tile_types[i].mining_time);
-	    }
-	}
-      else if (tile_types[i].mining_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].mining_result].terrain_name,
-		   tile_types[i].mining_time);
-	}
-      SetWindowText (help_tlabel[3][1], buf);
-
-      if (tile_types[i].transform_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].transform_result].terrain_name,
-		   tile_types[i].transform_time);
-	} else {
-	  strcpy (buf, "n/a");
-	}
-      SetWindowText (help_tlabel[3][4], buf);
+    if (*(tile_types[i].special_1_name)) {
+      sprintf(buf, _("%s F/R/T:"),
+	      tile_types[i].special_1_name);
+      SetWindowText(help_tlabel[1][0], buf);
+      sprintf(buf, "%d/%d/%d",
+	      tile_types[i].food_special_1,
+	      tile_types[i].shield_special_1,
+	      tile_types[i].trade_special_1);
+      SetWindowText(help_tlabel[1][1], buf);
+    } else {
+      SetWindowText(help_tlabel[1][0], " ");
+      SetWindowText(help_tlabel[1][1], " ");
     }
 
+    if (*(tile_types[i].special_2_name)) {
+      sprintf(buf, _("%s F/R/T:"),
+	      tile_types[i].special_2_name);
+      SetWindowText(help_tlabel[1][3], buf);
+      sprintf(buf, "%d/%d/%d",
+	      tile_types[i].food_special_2,
+	      tile_types[i].shield_special_2,
+	      tile_types[i].trade_special_2);
+      SetWindowText(help_tlabel[1][4], buf);
+    } else {
+      SetWindowText(help_tlabel[1][3], " ");
+      SetWindowText(help_tlabel[1][4], " ");
+    }
+
+    if (tile_types[i].road_trade_incr > 0) {
+      sprintf(buf, _("+%d Trade / %d"),
+	      tile_types[i].road_trade_incr,
+	      tile_types[i].road_time);
+    } else if (tile_types[i].road_time > 0) {
+      sprintf(buf, _("no extra / %d"),
+	      tile_types[i].road_time);
+    } else {
+      strcpy(buf, _("n/a"));
+    }
+    SetWindowText(help_tlabel[2][1], buf);
+
+    strcpy(buf, _("n/a"));
+    if (tile_types[i].irrigation_result == i) {
+      if (tile_types[i].irrigation_food_incr > 0) {
+	sprintf(buf, _("+%d Food / %d"),
+		tile_types[i].irrigation_food_incr,
+		tile_types[i].irrigation_time);
+      }
+    } else if (tile_types[i].irrigation_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      tile_types[tile_types[i].irrigation_result].terrain_name,
+	      tile_types[i].irrigation_time);
+    }
+    SetWindowText(help_tlabel[2][4], buf);
+
+    strcpy(buf, _("n/a"));
+    if (tile_types[i].mining_result == i) {
+      if (tile_types[i].mining_shield_incr > 0) {
+	sprintf(buf, _("+%d Res. / %d"),
+		tile_types[i].mining_shield_incr,
+		tile_types[i].mining_time);
+      }
+    } else if (tile_types[i].mining_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      tile_types[tile_types[i].mining_result].terrain_name,
+	      tile_types[i].mining_time);
+    }
+    SetWindowText(help_tlabel[3][1], buf);
+
+    if (tile_types[i].transform_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      tile_types[tile_types[i].transform_result].terrain_name,
+	      tile_types[i].transform_time);
+    } else {
+      strcpy(buf, "n/a");
+    }
+    SetWindowText(help_tlabel[3][4], buf);
+  }
+
   helptext_terrain(buf, i, pitem->text);
   set_help_text(buf);
-
 }
 
 /*************************************************************************
@@ -668,7 +687,7 @@ static void help_update_unit_type(const struct help_item *pitem,
   unit_num=i;
   if (i<game.num_unit_types) {
     struct unit_type *utype = get_unit_type(i);
-    sprintf(buf, "%d", utype->build_cost);
+    sprintf(buf, "%d", unit_build_shield_cost(i));
     SetWindowText(help_ulabel[0][1], buf);
     sprintf(buf, "%d", utype->attack_strength);
     SetWindowText(help_ulabel[0][4], buf);
@@ -731,10 +750,6 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 	  TECH_TREE_EXPANDED_DEPTH, NULL);
     */
     helptext_tech(buf, i, pitem->text);
-    if (advances[i].helptext) {
-      if (strlen(buf)) strcat(buf, "\n");
-      sprintf(buf+strlen(buf), "%s\n", _(advances[i].helptext));
-    }
     wordwrap_string(buf, 68);
     fcwin_box_add_static(helpdlg_page_vbox,buf,0,SS_LEFT,FALSE,FALSE,5);
 
@@ -769,7 +784,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 			   0,FALSE,FALSE,5);
     } unit_type_iterate_end;
 
-    for(j=0; j<game.num_tech_types; ++j) {
+    for (j = 0; j < game.num_tech_types; j++) {
       if(i==advances[j].req[0]) {
         if(advances[j].req[1]==A_NONE) {
 	  hbox=fcwin_hbox_new(helpdlg_win,FALSE);
@@ -832,7 +847,9 @@ static void help_update_dialog(const struct help_item *pitem)
   char *top;
   /* figure out what kind of item is required for pitem ingo */
 
-  for(top=pitem->topic; *top==' '; ++top);
+  for (top = pitem->topic; *top == ' '; top++) {
+    /* nothing */
+  }
   SetWindowText(helpdlg_text,"");
 
   switch(pitem->type) {
@@ -898,7 +915,7 @@ static void select_help_item_string(const char *item,
 
 **************************************************************************/
 void
-popup_help_dialog_string(char *item)
+popup_help_dialog_string(const char *item)
 {
   popup_help_dialog_typed(_(item), HELP_ANY); 
 }
@@ -907,7 +924,7 @@ popup_help_dialog_string(char *item)
 
 **************************************************************************/
 void
-popup_help_dialog_typed(char *item, enum help_page_type htype)
+popup_help_dialog_typed(const char *item, enum help_page_type htype)
 {
   if (!helpdlg_win)
     create_help_dialog();
