@@ -10,13 +10,12 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <stdlib.h>
-#include <string.h>
 
 #include "city.h"
 #include "fcintl.h"
@@ -24,7 +23,6 @@
 #include "government.h"
 #include "idex.h"
 #include "improvement.h"
-#include "log.h"
 #include "map.h"
 #include "mem.h"
 #include "rand.h"
@@ -35,6 +33,30 @@
 
 #include "player.h"
 
+/***************************************************************
+  Returns true iff p1 can ally p2. There is only one condition:
+  We are not at war with any of p2's allies. Note that for an
+  alliance to be made, we need to check this both ways.
+
+  The reason for this is to avoid the dread 'love-love-hate' 
+  triad, in which p1 is allied to p2 is allied to p3 is at
+  war with p1. These lead to strange situations.
+***************************************************************/
+bool pplayer_can_ally(struct player *p1, struct player *p2)
+{
+  players_iterate(pplayer) {
+    enum diplstate_type ds = pplayer_get_diplstate(p1, pplayer)->type;
+    if (pplayer != p1
+        && pplayer != p2
+        && pplayers_allied(p2, pplayer)
+        && ds == DS_WAR /* do not count 'never met' as war here */
+        && pplayer->is_alive) {
+      return FALSE;
+    }
+  } players_iterate_end;
+  return TRUE;
+}
+
 /***************************************************************
   Check if pplayer has an embassy with pplayer2. We always have
   an embassy with ourselves.
@@ -43,7 +65,7 @@ bool player_has_embassy(struct player *pplayer, struct player *pplayer2)
 {
   return (TEST_BIT(pplayer->embassy, pplayer2->player_no)
           || (pplayer == pplayer2)
-          || (player_owns_active_wonder(pplayer, B_MARCO)
+          || (get_player_bonus(pplayer, EFT_HAVE_EMBASSIES) > 0
               && !is_barbarian(pplayer2)));
 }
 
@@ -66,28 +88,34 @@ void player_init(struct player *plr)
 
   plr->player_no=plr-game.players;
 
-  sz_strlcpy(plr->name, "YourName");
-  sz_strlcpy(plr->username, "UserName");
+  sz_strlcpy(plr->name, ANON_PLAYER_NAME);
+  sz_strlcpy(plr->username, ANON_USER_NAME);
   plr->is_male = TRUE;
   plr->government=game.default_government;
-  plr->nation=MAX_NUM_NATIONS;
+  plr->nation = NO_NATION_SELECTED;
   plr->team = TEAM_NONE;
+  plr->is_started = FALSE;
+  plr->revolution_finishes = -1;
   plr->capital = FALSE;
   unit_list_init(&plr->units);
   city_list_init(&plr->cities);
   conn_list_init(&plr->connections);
   plr->current_conn = NULL;
   plr->is_connected = FALSE;
+  plr->is_observer = FALSE;
+  plr->was_created = FALSE;
   plr->is_alive=TRUE;
+  plr->is_dying = FALSE;
   plr->embassy=0;
   plr->reputation=GAME_DEFAULT_REPUTATION;
   for(i = 0; i < MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS; i++) {
     plr->diplstates[i].type = DS_NO_CONTACT;
     plr->diplstates[i].has_reason_to_cancel = 0;
+    plr->diplstates[i].contact_turns_left = 0;
   }
   plr->city_style=0;            /* should be first basic style */
   plr->ai.control=FALSE;
-  plr->ai.tech_goal = A_NONE;
+  plr->ai.tech_goal = A_UNSET;
   plr->ai.handicap = 0;
   plr->ai.skill_level = 0;
   plr->ai.fuzzy = 0;
@@ -106,69 +134,22 @@ void player_init(struct player *plr)
 
   /* Initialise list of improvements with Player-wide equiv_range */
   improvement_status_init(plr->improvements, ARRAY_SIZE(plr->improvements));
-  /* Initialise vector of effects with player range. */
-  geff_vector_init(&plr->effects);
 
-  /* Blank lists of Island-range improvements and effects (these are
-     initialised by player_init_island_impr) */
+  /* Initialise list of improvements with Island-wide equiv_range */
   plr->island_improv = NULL;
-  plr->island_effects = NULL;
+
+  if (map.num_continents > 0) {
+    plr->island_improv = fc_malloc((map.num_continents + 1) * 
+                                   game.num_impr_types * sizeof(Impr_Status));
+    for (i = 1; i <= map.num_continents; i++) {
+      improvement_status_init(&plr->island_improv[i * game.num_impr_types],
+                              game.num_impr_types);
+    }
+  }
 
   plr->attribute_block.data = NULL;
   plr->attribute_block.length = 0;
-}
-
-/***************************************************************
-  Set up the player's lists of Island-range improvements and
-  effects. These lists must also be redimensioned (e.g. by
-  update_island_impr_effect) if the number of islands later
-  changes.
-***************************************************************/
-void player_init_island_imprs(struct player *plr, int numcont)
-{
-  int i;
-
-  player_free_island_imprs(plr);
-  if (game.num_impr_types>0) {
-    /* Initialise lists of improvements with island-wide equiv_range. */
-    if (plr->island_improv)
-      free(plr->island_improv);
-    plr->island_improv=fc_calloc((numcont+1)*game.num_impr_types,
-				 sizeof(Impr_Status));
-    for (i=0; i<=numcont; i++) {
-      improvement_status_init(&plr->island_improv[i * game.num_impr_types],
-			      game.num_impr_types);
-    }
-
-    /* Initialise lists of island-range effects. */
-    plr->island_effects=fc_calloc(numcont+1, sizeof(struct geff_vector));
-    for (i=0; i<=numcont; i++) {
-      geff_vector_init(&plr->island_effects[i]);
-    }
-  }
-}
-
-/***************************************************************
-  Frees the player's list of island-range improvements and
-  effects.
-***************************************************************/
-void player_free_island_imprs(struct player *plr)
-{
-  int i;
-
-  if (plr->island_improv) {
-    free(plr->island_improv);
-    plr->island_improv = NULL;
-  }
-
-  if (plr->island_effects) {
-    for (i = 0; i <= plr->max_continent; i++) {
-      geff_vector_free(&plr->island_effects[i]);
-    }
-    free(plr->island_effects);
-    plr->island_effects = NULL;
-    plr->max_continent = 0;
-  }
+  plr->debug = FALSE;
 }
 
 /***************************************************************
@@ -204,6 +185,10 @@ struct player *find_player_by_name(const char *name)
 static const char *pname_accessor(int i) {
   return game.players[i].name;
 }
+
+/***************************************************************
+Find player by its name prefix
+***************************************************************/
 struct player *find_player_by_name_prefix(const char *name,
 					  enum m_pre_result *result)
 {
@@ -233,45 +218,108 @@ struct player *find_player_by_user(const char *name)
   return NULL;
 }
 
-/***************************************************************
-  Returns TRUE iff the player can see the unit at (x,y), i.e. if 
-  the unit is not invisible or if it is adjacent to one of our 
-  cities or units.  No map visibility check here! The unit does 
-  not have to be at (x,y)! 
-  Allied units and cities can be used for sub hunting. 
-***************************************************************/
-bool player_can_see_unit_at_location(struct player *pplayer, 
-                                     struct unit *punit, 
-                                     int x, int y)
+/****************************************************************************
+  Checks if a unit can be seen by pplayer at (x,y).
+  A player can see a unit if he:
+  (a) can see the tile AND
+  (b) can see the unit at the tile (i.e. unit not invisible at this tile) AND
+  (c) the unit is outside a city OR in an allied city AND
+  (d) the unit isn't in a transporter, or we are allied AND
+  (e) the unit isn't in a transporter, or we can see the transporter
+****************************************************************************/
+bool can_player_see_unit_at(struct player *pplayer, struct unit *punit,
+			    struct tile *ptile)
 {
+  struct city *pcity;
+
+  /* If the player can't even see the tile... */
+  if (map_get_known(ptile, pplayer) != TILE_KNOWN) {
+    return FALSE;
+  }
+
+  /* Don't show non-allied units that are in transports.  This is logical
+   * because allied transports can also contain our units.  Shared vision
+   * isn't taken into account. */
+  if (punit->transported_by != -1 && unit_owner(punit) != pplayer
+      && !pplayers_allied(pplayer, unit_owner(punit))) {
+    return FALSE;
+  }
+
+  /* Units in cities may be hidden. */
+  pcity = map_get_city(ptile);
+  if (pcity && !can_player_see_units_in_city(pplayer, pcity)) {
+    return FALSE;
+  }
+
+  /* Allied or non-hiding units are always seen. */
   if (pplayers_allied(unit_owner(punit), pplayer)
       || !is_hiding_unit(punit)) {
     return TRUE;
   }
 
-  /* Search for units/cities that might be able to see the sub/missile */
-  adjc_iterate(x, y, x1, y1) {
-    struct city *pcity = map_get_city(x1, y1);
-    unit_list_iterate(map_get_tile(x1, y1)->units, punit2) {
+  /* Hiding units may only be seen by adjacent allied units or cities. */
+  /* FIXME: shouldn't a check for shared vision be done here? */
+  adjc_iterate(ptile, ptile1) {
+    struct city *pcity = map_get_city(ptile1);
+    if (pcity && pplayers_allied(city_owner(pcity), pplayer)) {
+      return TRUE;
+    }  
+    unit_list_iterate(ptile1->units, punit2) {
       if (pplayers_allied(unit_owner(punit2), pplayer)) {
 	return TRUE;
       }
     } unit_list_iterate_end;
-
-    if (pcity && pplayers_allied(city_owner(pcity), pplayer)) {
-      return TRUE;
-    }
   } adjc_iterate_end;
 
   return FALSE;
 }
 
-/***************************************************************
-  Same thing as above only the location is the unit's current one.
-***************************************************************/
-bool player_can_see_unit(struct player *pplayer, struct unit *punit)
+
+/****************************************************************************
+  Checks if a unit can be seen by pplayer at its current location.
+
+  See can_player_see_unit_at.
+****************************************************************************/
+bool can_player_see_unit(struct player *pplayer, struct unit *punit)
 {
-  return player_can_see_unit_at_location(pplayer, punit, punit->x, punit->y);
+  return can_player_see_unit_at(pplayer, punit, punit->tile);
+}
+
+/****************************************************************************
+  Return TRUE iff the player can see units in the city.  Either they
+  can see all units or none.
+
+  If the player can see units in the city, then the server sends the
+  unit info for units in the city to the client.  The client uses the
+  tile's unitlist to determine whether to show the city occupied flag.  Of
+  course the units will be visible to the player as well, if he clicks on
+  them.
+
+  If the player can't see units in the city, then the server doesn't send
+  the unit info for these units.  The client therefore uses the "occupied"
+  flag sent in the short city packet to determine whether to show the city
+  occupied flag.
+
+  Note that can_player_see_city_internals => can_player_see_units_in_city.
+  Otherwise the player would not know anything about the city's units at
+  all, since the full city packet has no "occupied" flag.
+****************************************************************************/
+bool can_player_see_units_in_city(struct player *pplayer,
+				  struct city *pcity)
+{
+  return (can_player_see_city_internals(pplayer, pcity)
+	  || pplayers_allied(pplayer, city_owner(pcity)));
+}
+
+/****************************************************************************
+  Return TRUE iff the player can see the city's internals.  This means the
+  full city packet is sent to the client, who should then be able to popup
+  a dialog for it.
+****************************************************************************/
+bool can_player_see_city_internals(struct player *pplayer,
+				   struct city *pcity)
+{
+  return (pplayer == city_owner(pcity));
 }
 
 /***************************************************************
@@ -279,7 +327,8 @@ bool player_can_see_unit(struct player *pplayer, struct unit *punit)
  return pointer to the city struct.  Else return NULL.
  Now always uses fast idex_lookup_city.
 ***************************************************************/
-struct city *player_find_city_by_id(struct player *pplayer, int city_id)
+struct city *player_find_city_by_id(const struct player *pplayer,
+				    int city_id)
 {
   struct city *pcity = idex_lookup_city(city_id);
   
@@ -295,7 +344,8 @@ struct city *player_find_city_by_id(struct player *pplayer, int city_id)
  return pointer to the unit struct.  Else return NULL.
  Uses fast idex_lookup_city.
 ***************************************************************/
-struct unit *player_find_unit_by_id(struct player *pplayer, int unit_id)
+struct unit *player_find_unit_by_id(const struct player *pplayer,
+				    int unit_id)
 {
   struct unit *punit = idex_lookup_unit(unit_id);
   
@@ -309,42 +359,17 @@ struct unit *player_find_unit_by_id(struct player *pplayer, int unit_id)
 /*************************************************************************
 Return 1 if x,y is inside any of the player's city radii.
 **************************************************************************/
-bool player_in_city_radius(struct player *pplayer, int x, int y)
+bool player_in_city_radius(struct player *pplayer, struct tile *ptile)
 {
   struct city *pcity;
-  map_city_radius_iterate(x, y, x1, y1) {
-    pcity = map_get_city(x1, y1);
+  map_city_radius_iterate(ptile, ptile1) {
+    pcity = map_get_city(ptile1);
     if (pcity && (pcity->owner == pplayer->player_no))
       return TRUE;
   } map_city_radius_iterate_end;
   return FALSE;
 }
 
-/**************************************************************************
- Return 1 if one of the player's cities has the specified wonder,
- and it is not obsolete.
-**************************************************************************/
-bool player_owns_active_wonder(struct player *pplayer,
-			      Impr_Type_id id)
-{
-  return (improvement_exists(id)
-	  && is_wonder(id)
-	  && (!wonder_obsolete(id))
-	  && player_find_city_by_id(pplayer, game.global_wonders[id]));
-}
-
-/**************************************************************************
- ...
-**************************************************************************/
-bool player_owns_active_govchange_wonder(struct player *pplayer)
-{
-  return ( player_owns_active_wonder(pplayer, B_LIBERTY) ||
-	   ( (improvement_variant(B_PYRAMIDS)==1) &&
-	     player_owns_active_wonder(pplayer, B_PYRAMIDS) ) ||
-	   ( (improvement_variant(B_UNITED)==1) &&
-	     player_owns_active_wonder(pplayer, B_UNITED) ) );
-}
-
 /**************************************************************************
  Returns the number of techs the player has researched which has this
  flag. Needs to be optimized later (e.g. int tech_flags[TF_LAST] in
@@ -355,6 +380,31 @@ int num_known_tech_with_flag(struct player *pplayer, enum tech_flag_id flag)
   return pplayer->research.num_known_tech_with_flag[flag];
 }
 
+/**************************************************************************
+  Return the expected net income of the player this turn.  This includes
+  tax revenue and upkeep, but not one-time purchases or found gold.
+
+  This function depends on pcity->tax_total being set for all cities, so
+  make sure the player's cities have been refreshed.
+**************************************************************************/
+int player_get_expected_income(struct player *pplayer)
+{
+  int income = 0;
+
+  /* City income/expenses. */
+  city_list_iterate(pplayer->cities, pcity) {
+    /* Gold suplus accounts for imcome plus building and unit upkeep. */
+    income += city_gold_surplus(pcity, pcity->tax_total);
+
+    /* Capitalization income. */
+    if (get_current_construction_bonus(pcity, EFT_PROD_TO_GOLD) > 0) {
+      income += pcity->shield_stock + pcity->shield_surplus;
+    }
+  } city_list_iterate_end;
+
+  return income;
+}
+
 /**************************************************************************
  Returns TRUE iff the player knows at least one tech which has the
  given flag.
@@ -405,7 +455,7 @@ void player_limit_to_government_rates(struct player *pplayer)
     } else if (pplayer->economic.luxury < maxrate) {
       pplayer->economic.luxury += 10;
     } else {
-      abort();
+      die("byebye");
     }
     surplus -= 10;
   }
@@ -413,35 +463,16 @@ void player_limit_to_government_rates(struct player *pplayer)
   return;
 }
 
-/**************************************************************************
-  Hack to support old code which expects player to have single address
-  string (possibly blank_addr_str) and not yet updated for multi-connects.
-  When all such code is converted, this function should be removed.
-  Returns "blank" address for no connections, single address for one,
-  or string "<multiple>" if there are multiple connections.
-  
-  Currently used by:
-      civserver.c: send_server_info_to_metaserver()
-      plrhand.c: send_player_info_c()
-      gui-{xaw,gtk,mui}/plrdlg.c: update_players_dialog()
-**************************************************************************/
-const char *player_addr_hack(struct player *pplayer)
-{
-  int n = conn_list_size(&pplayer->connections);
-  return ((n==0) ? blank_addr_str :
-	  (n==1) ? conn_list_get(&pplayer->connections, 0)->addr :
-	  "<multiple>");
-}
-
 /**************************************************************************
 Locate the city where the players palace is located, (NULL Otherwise) 
 **************************************************************************/
 struct city *find_palace(struct player *pplayer)
 {
-  city_list_iterate(pplayer->cities, pcity) 
-    if (city_got_building(pcity, B_PALACE)) 
+  city_list_iterate(pplayer->cities, pcity) {
+    if (is_capital(pcity)) {
       return pcity;
-  city_list_iterate_end;
+    }
+  } city_list_iterate_end;
   return NULL;
 }
 
@@ -491,6 +522,42 @@ bool ai_fuzzy(struct player *pplayer, bool normal_decision)
   return !normal_decision;
 }
 
+
+
+/**************************************************************************
+  Return a text describing an AI's love for you.  (Oooh, kinky!!)
+  These words should be adjectives which can fit in the sentence
+  "The x are y towards us"
+  "The Babylonians are respectful towards us"
+**************************************************************************/
+const char *love_text(const int love)
+{
+  if (love <= - MAX_AI_LOVE * 90 / 100) {
+    return Q_("?attitude:Genocidal");
+  } else if (love <= - MAX_AI_LOVE * 70 / 100) {
+    return Q_("?attitude:Belligerent");
+  } else if (love <= - MAX_AI_LOVE * 50 / 100) {
+    return Q_("?attitude:Hostile");
+  } else if (love <= - MAX_AI_LOVE * 25 / 100) {
+    return Q_("?attitude:Uncooperative");
+  } else if (love <= - MAX_AI_LOVE * 10 / 100) {
+    return Q_("?attitude:Uneasy");
+  } else if (love <= MAX_AI_LOVE * 10 / 100) {
+    return Q_("?attitude:Neutral");
+  } else if (love <= MAX_AI_LOVE * 25 / 100) {
+    return Q_("?attitude:Respectful");
+  } else if (love <= MAX_AI_LOVE * 50 / 100) {
+    return Q_("?attitude:Helpful");
+  } else if (love <= MAX_AI_LOVE * 70 / 100) {
+    return Q_("?attitude:Enthusiastic");
+  } else if (love <= MAX_AI_LOVE * 90 / 100) {
+    return Q_("?attitude:Admiring");
+  } else {
+    assert(love > MAX_AI_LOVE * 90 / 100);
+    return Q_("?attitude:Worshipful");
+  }
+}
+
 /**************************************************************************
 Return a reputation level as a human-readable string
 **************************************************************************/
@@ -517,7 +584,7 @@ const char *reputation_text(const int rep)
 }
 
 /**************************************************************************
-Return a diplomatic state as a human-readable string
+  Return a diplomatic state as a human-readable string
 **************************************************************************/
 const char *diplstate_text(const enum diplstate_type type)
 {
@@ -528,13 +595,15 @@ const char *diplstate_text(const enum diplstate_type type)
     N_("?diplomatic_state:Cease-fire"),
     N_("?diplomatic_state:Peace"),
     N_("?diplomatic_state:Alliance"),
-    N_("?diplomatic_state:No Contact")
+    N_("?diplomatic_state:Never met"),
+    N_("?diplomatic_state:Team")
   };
 
-  if (type < DS_LAST)
+  if (type < DS_LAST) {
     return Q_(ds_names[type]);
-  freelog(LOG_FATAL, "Bad diplstate_type in diplstate_text: %d", type);
-  abort();
+  }
+  die("Bad diplstate_type in diplstate_text: %d", type);
+  return NULL;
 }
 
 /***************************************************************
@@ -547,9 +616,58 @@ const struct player_diplstate *pplayer_get_diplstate(const struct player *pplaye
 }
 
 /***************************************************************
-  Returns true iff players have no contact status.
+  Returns true iff players can attack each other.
 ***************************************************************/
-bool pplayers_no_contact(const struct player *pplayer,
+bool pplayers_at_war(const struct player *pplayer,
+                     const struct player *pplayer2)
+{
+  enum diplstate_type ds = pplayer_get_diplstate(pplayer, pplayer2)->type;
+  if (pplayer == pplayer2) {
+    return FALSE;
+  }
+  if (is_barbarian(pplayer) || is_barbarian(pplayer2)) {
+    return TRUE;
+  }
+  return ds == DS_WAR || ds == DS_NO_CONTACT;
+}
+
+/***************************************************************
+  Returns true iff players are allied.
+***************************************************************/
+bool pplayers_allied(const struct player *pplayer,
+                     const struct player *pplayer2)
+{
+  enum diplstate_type ds = pplayer_get_diplstate(pplayer, pplayer2)->type;
+  if (pplayer == pplayer2) {
+    return TRUE;
+  }
+  if (is_barbarian(pplayer) || is_barbarian(pplayer2)) {
+    return FALSE;
+  }
+  return (ds == DS_ALLIANCE || ds == DS_TEAM);
+}
+
+/***************************************************************
+  Returns true iff players are allied or at peace.
+***************************************************************/
+bool pplayers_in_peace(const struct player *pplayer,
+                       const struct player *pplayer2)
+{
+  enum diplstate_type ds = pplayer_get_diplstate(pplayer, pplayer2)->type;
+
+  if (pplayer == pplayer2) {
+    return TRUE;
+  }
+  if (is_barbarian(pplayer) || is_barbarian(pplayer2)) {
+    return FALSE;
+  }
+  return (ds == DS_PEACE || ds == DS_ALLIANCE || ds == DS_TEAM);
+}
+
+/***************************************************************
+  Returns true iff players have peace or cease-fire.
+***************************************************************/
+bool pplayers_non_attack(const struct player *pplayer,
                          const struct player *pplayer2)
 {
   enum diplstate_type ds = pplayer_get_diplstate(pplayer, pplayer2)->type;
@@ -559,53 +677,21 @@ bool pplayers_no_contact(const struct player *pplayer,
   if (is_barbarian(pplayer) || is_barbarian(pplayer2)) {
     return FALSE;
   }
-  return ds == DS_NO_CONTACT;
-}
-
-/***************************************************************
-returns true iff players can attack each other.
-***************************************************************/
-bool pplayers_at_war(const struct player *pplayer,
-		    const struct player *pplayer2)
-{
-  enum diplstate_type ds = pplayer_get_diplstate(pplayer, pplayer2)->type;
-  if (pplayer == pplayer2) return FALSE;
-  if (is_barbarian(pplayer) || is_barbarian(pplayer2))
-    return TRUE;
-  return ds == DS_WAR || ds == DS_NO_CONTACT;
-}
-
-/***************************************************************
-returns true iff players are allied
-***************************************************************/
-bool pplayers_allied(const struct player *pplayer,
-		    const struct player *pplayer2)
-{
-  enum diplstate_type ds = pplayer_get_diplstate(pplayer, pplayer2)->type;
-  if (pplayer == pplayer2)
-    return TRUE;
-  if (is_barbarian(pplayer) || is_barbarian(pplayer2))
-    return FALSE;
-  return (ds == DS_ALLIANCE);
-}
-
-/***************************************************************
-returns true iff players have peace or cease-fire
-***************************************************************/
-bool pplayers_non_attack(const struct player *pplayer,
-			const struct player *pplayer2)
-{
-  enum diplstate_type ds = pplayer_get_diplstate(pplayer, pplayer2)->type;
-  if (pplayer == pplayer2)
-    return FALSE;
-  if (is_barbarian(pplayer) || is_barbarian(pplayer2))
-    return FALSE;
   return (ds == DS_PEACE || ds == DS_CEASEFIRE || ds == DS_NEUTRAL);
 }
 
 /**************************************************************************
-...
+  Return TRUE if players are in the same team
 **************************************************************************/
+bool players_on_same_team(const struct player *pplayer1,
+                          const struct player *pplayer2)
+{
+  if (pplayer1->team == TEAM_NONE) {
+    return FALSE;
+  }
+  return (pplayer1->team == pplayer2->team);
+}
+
 bool is_barbarian(const struct player *pplayer)
 {
   return pplayer->ai.barbarian_type != NOT_A_BARBARIAN;
@@ -618,3 +704,56 @@ bool gives_shared_vision(struct player *me, struct player *them)
 {
   return TEST_BIT(me->gives_shared_vision, them->player_no);
 }
+
+/**************************************************************************
+  Return TRUE iff the two diplstates are equal.
+**************************************************************************/
+bool are_diplstates_equal(const struct player_diplstate *pds1,
+			  const struct player_diplstate *pds2)
+{
+  return (pds1->type == pds2->type && pds1->turns_left == pds2->turns_left
+	  && pds1->has_reason_to_cancel == pds2->has_reason_to_cancel
+	  && pds1->contact_turns_left == pds2->contact_turns_left);
+}
+
+/***************************************************************************
+  Return the number of pplayer2's visible units in pplayer's territory,
+  from the point of view of pplayer.  Units that cannot be seen by pplayer
+  will not be found (this function doesn't cheat).
+***************************************************************************/
+int player_in_territory(struct player *pplayer, struct player *pplayer2)
+{
+  int in_territory = 0;
+
+  /* This algorithm should work at server or client.  It only returns the
+   * number of visible units (a unit can potentially hide inside the
+   * transport of a different player).
+   *
+   * Note this may be quite slow.  An even slower alternative is to iterate
+   * over the entire map, checking all units inside the player's territory
+   * to see if they're owned by the enemy. */
+  unit_list_iterate(pplayer2->units, punit) {
+    /* Get the owner of the tile/territory. */
+    struct player *owner = map_get_owner(punit->tile);
+
+    if (owner == pplayer && can_player_see_unit(pplayer, punit)) {
+      /* Found one! */
+      in_territory += 1;
+    }
+  } unit_list_iterate_end;
+
+  return in_territory;
+}
+
+/****************************************************************************
+  Returns whether this is a valid username.  This is used by the server to
+  validate usernames and should be used by the client to avoid invalid
+  ones.
+****************************************************************************/
+bool is_valid_username(const char *name)
+{
+  return (strlen(name) > 0
+	  && !my_isdigit(name[0])
+	  && is_ascii_name(name)
+	  && mystrcasecmp(name, ANON_USER_NAME) != 0);
+}
